/*
 * Copyright (c) 2004 on behalf of the EU EGEE Project:
 * The European Organization for Nuclear Research (CERN),
 * Istituto Nazionale di Fisica Nucleare (INFN), Italy
 * Datamat Spa, Italy
 * Centre National de la Recherche Scientifique (CNRS), France
 * CS Systeme d'Information (CSSI), France
 * Royal Institute of Technology, Center for Parallel Computers (KTH-PDC), Sweden
 * Universiteit van Amsterdam (UvA), Netherlands
 * University of Helsinki (UH.HIP), Finland
 * University of Bergen (UiB), Norway
 * Council for the Central Laboratory of the Research Councils (CCLRC), United Kingdom
 *
 * Authors: Paolo Badino <Paolo.Badino@cern.ch>
 *
 * Version info: $Id: QUANTAnet_null_auth_c.hxx,v 1.5 2004/08/09 09:40:57 badino Exp $
 *
 * Revision history:
 * $ log$
 *
 */
 
#ifndef _QUANTAPLUS_QUANTAPLUS_NULL_AUTH_CLASS
#define _QUANTAPLUS_QUANTAPLUS_NULL_AUTH_CLASS

#include "glite/data/io/quanta/QUANTAnet_auth_strategy_c.hxx"

/**
 * QUANTAnet_null_auth_context_c
 * Fake class fot authorization Context
 */
class QUANTAnet_null_auth_context_c : public QUANTAnet_auth_context_c{

public:
    /**
     * Constructor
     */
    QUANTAnet_null_auth_context_c(){}
    /**
     * Destructor
     */
    virtual ~QUANTAnet_null_auth_context_c(){}   

	/**
     * Return User's Name
     * @return User Name or 0  if not available
     */
    virtual const char * getName(){ return "";}

    /**
     * Store User's Credentials in a file
     * @param filename [IN] the name of the file where to store the user 
     * credentials
     * @return 0 if success
     */
    virtual int exportCredentials(const char * filename){return -1;}
   
    /**
     * Export User's Credentials to a buffer. Buffer Should be deallocated by 
     * calling QUANTAnet_auth_context_c::releaseBuffer
     * @return 0 if success
     */
    virtual int exportCredentials(char ** buffer, int * buf_len){return -1;}
   
    /**
     * Check the if a Protection Mode is enabled
     * @param pm [IN] the protection mode to check
     * @return true if the requested protection service is available
     */
    virtual bool checkProtectionMode(ProtectionMode pm){return (pm == PM_NONE);}
   
	/**
     * Encrypt Data according the the requested Protection Mode - Do Nothing
     * @param src_buffer [IN] the buffer to encrypt
     * @param src_len [IN] the length of the source buffer
     * @param dst_buffer [OUT] the destination buffer. The buffer is allocated 
     *   internally and should be freed using the method @see releaseBuffer
     * @param dst_len [OUT] the length of the destination buffer
     * @return always -1
     */
    virtual int encrypt(const char * src_buffer, int src_len, char * &dst_buffer, int &dst_len, ProtectionMode pm = PM_PRIVACY){return -1;}
	
    /**
     * Decrypt Data - Do Nothing
     * @param src_buffer [IN] the buffer to decrypt
     * @param src_len [IN] the length of the source buffer
     * @param dst_buffer [OUT] the destination buffer. The buffer is allocated 
     *   internally and should be freed using the method @see releaseBuffer
     * @param dst_len [OUT] the length of the destination buffer
     * @return always -1
     */
    virtual int decrypt(const char * src_buffer, int src_len, char * &dst_buffer, int &dst_len){return -1;}
    
    /**
     * Release a buffer previosly allocated by @see encrypt or @see descrypt - Do Nothing
     * @param buffer [IN] the buffer to release
     */
    virtual void releaseBuffer(char * &buffer, int buffer_len){}
};

/**
 * QUANTAnet_null_client_auth_strategy_c
 * Fake class for Client authorization Context Factory
 */
class QUANTAnet_null_client_auth_strategy_c : public QUANTAnet_client_auth_strategy_c {

public:
    /**
     * Constructor
     */
    QUANTAnet_null_client_auth_strategy_c() {}    
    
    /**
     * Destructor
     */
    virtual ~QUANTAnet_null_client_auth_strategy_c(){}
   
    /**
     * Initialize
     * @return always 0
     */
    virtual int init(){return 0;}
    
    /**
     * Enable Encryption Service
     * @return always -1
     */
    virtual int enableEncryption(){ return -1;}
    
    /**
     * Enable Delegation Service
     * @return always -1
     */
    virtual int enableDelegation(){ return -1;}
    
    /**
     * Import the Credentials for Delegation from a proxy file
     * @param filename [IN] the file containing the credentials to be used
     * for delegation. This parameters is returned by a call to 
     * QUANTAnet_auth_context_c::exportCredentials
     * @return 0 if success
     */
    virtual int importCredentials(const char * filename){return -1;}
    
    /**
     * Import the Credentials for Delegation from a buffer created by a call to
     * QUANTAnet_auth_context_c::exportCredentials
     * @param buffer [IN] the buffer containing the credentials to be used
     * for delegation
     * @param buf_len [IN] the length of the buffer
     * @return 0 if success
     */
    virtual int importCredentials(const char * buffer, int buf_len){return -1;}
    
    /**
     * Establish the Authorization Context 
     * @param sock_handle [IN] handle of the socket used to contact the peer
     * @return a null Security Context
     */
    virtual QUANTAnet_auth_context_c * establish(int sock_handle){return new QUANTAnet_null_auth_context_c();}
    
};

/**
 * QUANTAnet_gss_server_auth_strategy_c
 * GSS authorization Context Server Factory
 */
class QUANTAnet_null_server_auth_strategy_c: public QUANTAnet_server_auth_strategy_c {
  
public:
	/**
     * Constructor
     */
    QUANTAnet_null_server_auth_strategy_c() {}	
	
    /**
     * Destructor
     */
    virtual ~QUANTAnet_null_server_auth_strategy_c(){}
	
	/**
     * Initialize
     * @return 0 if success
     */
    virtual int init(){return 0;}
    
    /**
     * Accept a Security Context 
     * @param sock_handle [IN] handle of the socket used to contact the peer
     * @return the fake Security Context
     */
    virtual QUANTAnet_auth_context_c * accept(int sock_handle){return new QUANTAnet_null_auth_context_c();}
    
    /**
     * Authorize an established Security Context 
     * @param ctx [IN] the security context to be authorized
     * @return always 0 (success)
     */
    virtual int authorize(QUANTAnet_auth_context_c * ctx){return 0;}
    
};

#endif //_QUANTAPLUS_QUANTAPLUS_NULL_AUTH_CLASS
